@@ -507,11 +507,26 @@ snapshot_state_to_qcdm (u_int8_t cdma_state)
     return cdma_state + 1;
 }
 
+static inline u_int8_t
+digit_fixup (u_int8_t d)
+{
+    /* CDMA MCC/IMSI conversion adds 1 to each digit, and digits equal to
+     * 10 are really zero.
+     */
+    if (d + 1 < 10)
+        return d + 1;
+    return 0;
+}
+
 QcdmResult *
 qcdm_cmd_status_snapshot_result (const char *buf, size_t len, int *out_error)
 {
     QcdmResult *result = NULL;
     DMCmdStatusSnapshotRsp *rsp = (DMCmdStatusSnapshotRsp *) buf;
+    char *tmp;
+    u_int8_t swapped[4];
+    u_int8_t tmcc[3];
+    u_int16_t mcc, hmcc;
 
     qcdm_return_val_if_fail (buf != NULL, NULL);
 
@@ -520,6 +535,27 @@ qcdm_cmd_status_snapshot_result (const char *buf, size_t len, int *out_error)
 
     result = qcdm_result_new ();
 
+    /* Convert the ESN from binary to a hex string; it's LE so we have to
+     * swap it to get the correct ordering.
+     */
+    swapped[0] = rsp->esn[3];
+    swapped[1] = rsp->esn[2];
+    swapped[2] = rsp->esn[1];
+    swapped[3] = rsp->esn[0];
+
+    tmp = bin2hexstr (&swapped[0], sizeof (swapped));
+    qcdm_result_add_string (result, QCDM_CMD_STATUS_SNAPSHOT_ITEM_ESN, tmp);
+    free (tmp);
+
+    /* Cheap binary -> decimal conversion */
+    hmcc = le16toh (rsp->mcc);
+    tmcc[2] = hmcc / 100;
+    tmcc[1] = (hmcc - (tmcc[2] * 100)) / 10;
+    tmcc[0] = (hmcc - (tmcc[2] * 100) - (tmcc[1] * 10));
+
+    mcc = (100 * digit_fixup (tmcc[2])) + (10 * digit_fixup (tmcc[1])) + digit_fixup (tmcc[0]);
+    qcdm_result_add_u32 (result, QCDM_CMD_STATUS_SNAPSHOT_ITEM_HOME_MCC, mcc);
+
     qcdm_result_add_u8 (result, QCDM_CMD_STATUS_SNAPSHOT_ITEM_BAND_CLASS, cdma_band_class_to_qcdm (rsp->band_class));
     qcdm_result_add_u8 (result, QCDM_CMD_STATUS_SNAPSHOT_ITEM_BASE_STATION_PREV, cdma_prev_to_qcdm (rsp->prev));
     qcdm_result_add_u8 (result, QCDM_CMD_STATUS_SNAPSHOT_ITEM_MOBILE_PREV, cdma_prev_to_qcdm (rsp->mob_prev));
@@ -923,6 +959,94 @@ qcdm_cmd_nv_set_mode_pref_result (const char *buf, size_t len, int *out_error)
 
 /**********************************************************************/
 
+size_t
+qcdm_cmd_nv_get_hybrid_pref_new (char *buf, size_t len)
+{
+    char cmdbuf[sizeof (DMCmdNVReadWrite) + 2];
+    DMCmdNVReadWrite *cmd = (DMCmdNVReadWrite *) &cmdbuf[0];
+
+    qcdm_return_val_if_fail (buf != NULL, 0);
+    qcdm_return_val_if_fail (len >= sizeof (*cmd) + DIAG_TRAILER_LEN, 0);
+
+    memset (cmd, 0, sizeof (*cmd));
+    cmd->code = DIAG_CMD_NV_READ;
+    cmd->nv_item = htole16 (DIAG_NV_HYBRID_PREF);
+
+    return dm_encapsulate_buffer (cmdbuf, sizeof (*cmd), sizeof (cmdbuf), buf, len);
+}
+
+QcdmResult *
+qcdm_cmd_nv_get_hybrid_pref_result (const char *buf, size_t len, int *out_error)
+{
+    QcdmResult *result = NULL;
+    DMCmdNVReadWrite *rsp = (DMCmdNVReadWrite *) buf;
+    DMNVItemHybridPref *hybrid;
+
+    qcdm_return_val_if_fail (buf != NULL, NULL);
+
+    if (!check_command (buf, len, DIAG_CMD_NV_READ, sizeof (DMCmdNVReadWrite), out_error))
+        return NULL;
+
+    if (!check_nv_cmd (rsp, DIAG_NV_HYBRID_PREF, out_error))
+        return NULL;
+
+    hybrid = (DMNVItemHybridPref *) &rsp->data[0];
+
+    if (hybrid->hybrid_pref > 1)
+        qcdm_warn (0, "Unknown hybrid preference 0x%X", hybrid->hybrid_pref);
+
+    result = qcdm_result_new ();
+    qcdm_result_add_u8 (result, QCDM_CMD_NV_GET_HYBRID_PREF_ITEM_HYBRID_PREF, hybrid->hybrid_pref);
+
+    return result;
+}
+
+size_t
+qcdm_cmd_nv_set_hybrid_pref_new (char *buf,
+                                 size_t len,
+                                 u_int8_t hybrid_pref)
+{
+    char cmdbuf[sizeof (DMCmdNVReadWrite) + 2];
+    DMCmdNVReadWrite *cmd = (DMCmdNVReadWrite *) &cmdbuf[0];
+    DMNVItemHybridPref *req;
+
+    qcdm_return_val_if_fail (buf != NULL, 0);
+    qcdm_return_val_if_fail (len >= sizeof (*cmd) + DIAG_TRAILER_LEN, 0);
+
+    if (hybrid_pref > QCDM_CMD_NV_HYBRID_PREF_ITEM_REV_HYBRID_ON) {
+        qcdm_err (0, "Invalid hybrid preference %d", hybrid_pref);
+        return 0;
+    }
+
+    memset (cmd, 0, sizeof (*cmd));
+    cmd->code = DIAG_CMD_NV_WRITE;
+    cmd->nv_item = htole16 (DIAG_NV_HYBRID_PREF);
+
+    req = (DMNVItemHybridPref *) &cmd->data[0];
+    if (hybrid_pref == QCDM_CMD_NV_HYBRID_PREF_ITEM_REV_HYBRID_OFF)
+        req->hybrid_pref = DIAG_NV_HYBRID_PREF_OFF;
+    else if (hybrid_pref == QCDM_CMD_NV_HYBRID_PREF_ITEM_REV_HYBRID_ON)
+        req->hybrid_pref = DIAG_NV_HYBRID_PREF_ON;
+
+    return dm_encapsulate_buffer (cmdbuf, sizeof (*cmd), sizeof (cmdbuf), buf, len);
+}
+
+QcdmResult *
+qcdm_cmd_nv_set_hybrid_pref_result (const char *buf, size_t len, int *out_error)
+{
+    qcdm_return_val_if_fail (buf != NULL, NULL);
+
+    if (!check_command (buf, len, DIAG_CMD_NV_WRITE, sizeof (DMCmdNVReadWrite), out_error))
+        return NULL;
+
+    if (!check_nv_cmd ((DMCmdNVReadWrite *) buf, DIAG_NV_HYBRID_PREF, out_error))
+        return NULL;
+
+    return qcdm_result_new ();
+}
+
+/**********************************************************************/
+
 static qcdmbool
 hdr_rev_pref_validate (u_int8_t dm)
 {
