@@ -461,9 +461,8 @@ get_act_request_done (MMAtSerialPort *port,
     }
 
     if (!g_regex_match_full (r, response->str, response->len, 0, 0, &match_info, &info->error)) {
-        g_set_error_literal (&info->error,
-                             MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
-                             "Could not parse ^SYSINFO results.");
+        g_prefix_error (&info->error,
+                        "Could not parse ^SYSINFO results: ");
         goto done;
     }
 
@@ -561,9 +560,8 @@ send_huawei_cpin_done (MMAtSerialPort *port,
     }
 
     if (!g_regex_match_full (r, response->str, response->len, 0, 0, &match_info, &info->error)) {
-        g_set_error_literal (&info->error,
-                             MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
-                             "Could not parse ^CPIN results (match failed).");
+        g_prefix_error (&info->error,
+                        "Could not parse ^CPIN results (match failed): ");
         goto done;
     }
 
@@ -702,7 +700,7 @@ handle_mode_change (MMAtSerialPort *port,
     } else if (a == 0)
         act = MM_MODEM_GSM_ACCESS_TECH_UNKNOWN;
     else {
-        mm_warn ("Couldn't parse mode change value: '%s'", str);
+        mm_warn ("Couldn't parse mode change value: '%d'", a);
         return;
     }
 
@@ -793,6 +791,10 @@ grab_port (MMModem *modem,
         regex = g_regex_new ("\\r\\n\\^BOOT:.+\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
         mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, NULL, modem, NULL);
         g_regex_unref (regex);
+
+        regex = g_regex_new ("\\r\\n\\^CSNR:.+\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, NULL, modem, NULL);
+        g_regex_unref (regex);
     }
 
 out:
@@ -897,4 +899,3 @@ mm_modem_huawei_gsm_class_init (MMModemHuaweiGsmClass *klass)
     gsm_class->get_allowed_mode = get_allowed_mode;
     gsm_class->get_access_technology = get_access_technology;
 }
-
