@@ -912,6 +912,7 @@ disconnect_flash_done (MMSerialPort *port,
 
 static void
 disconnect (MMModem *modem,
+            MMModemStateReason reason,
             MMModemFn callback,
             gpointer user_data)
 {
@@ -930,7 +931,7 @@ disconnect (MMModem *modem,
                                GUINT_TO_POINTER (state),
                                NULL);
 
-    mm_modem_set_state (modem, MM_MODEM_STATE_DISCONNECTING, MM_MODEM_STATE_REASON_NONE);
+    mm_modem_set_state (modem, MM_MODEM_STATE_DISCONNECTING, reason);
     mm_serial_port_flash (MM_SERIAL_PORT (priv->primary), 1000, TRUE, disconnect_flash_done, info);
 }
 
@@ -1857,9 +1858,9 @@ reg_hdrstate_cb (MMQcdmSerialPort *port,
         guint8 almp_state = QCDM_CMD_HDR_SUBSYS_STATE_INFO_ALMP_STATE_INACTIVE;
         guint8 hybrid_mode = 0;
 
-        if (   qcdm_result_get_u8 (result, QCDM_CMD_HDR_SUBSYS_STATE_INFO_ITEM_SESSION_STATE, &session_state)
-            && qcdm_result_get_u8 (result, QCDM_CMD_HDR_SUBSYS_STATE_INFO_ITEM_ALMP_STATE, &almp_state)
-            && qcdm_result_get_u8 (result, QCDM_CMD_HDR_SUBSYS_STATE_INFO_ITEM_HDR_HYBRID_MODE, &hybrid_mode)) {
+        if (   qcdm_result_get_u8 (result, QCDM_CMD_HDR_SUBSYS_STATE_INFO_ITEM_SESSION_STATE, &session_state) == 0
+            && qcdm_result_get_u8 (result, QCDM_CMD_HDR_SUBSYS_STATE_INFO_ITEM_ALMP_STATE, &almp_state) == 0
+            && qcdm_result_get_u8 (result, QCDM_CMD_HDR_SUBSYS_STATE_INFO_ITEM_HDR_HYBRID_MODE, &hybrid_mode) == 0) {
 
             /* EVDO state is registered if the HDR subsystem is registered, and
              * we're in hybrid mode, and the Call Manager system mode is
