@@ -249,6 +249,8 @@ get_ip4_invoke (MMCallbackInfo *info)
 
     callback (info->modem,
               GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-address")),
+              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-netmask")),
+              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-gateway")),
               (GArray *) mm_callback_info_get_data (info, "ip4-dns"),
               info->error, info->user_data);
 }
@@ -291,6 +293,8 @@ value_array_add_uint (GValueArray *array, guint32 i)
 static void
 get_ip4_done (MMModem *modem,
               guint32 address,
+              guint32 netmask,
+              guint32 gateway,
               GArray *dns,
               GError *error,
               gpointer user_data)
@@ -309,7 +313,6 @@ get_ip4_done (MMModem *modem,
 
         if (dns) {
             if (dns->len > 0)
-
                 dns1 = g_array_index (dns, guint32, 0);
             if (dns->len > 1)
                 dns2 = g_array_index (dns, guint32, 1);
@@ -738,6 +741,7 @@ mm_modem_set_state (MMModem *self,
     if (new_state != old_state) {
         g_object_set (G_OBJECT (self), MM_MODEM_STATE, new_state, NULL);
         g_signal_emit_by_name (G_OBJECT (self), "state-changed", old_state, new_state, reason);
+        g_object_notify (G_OBJECT (self), MM_MODEM_STATE);
 
         dbus_path = (const char *) g_object_get_data (G_OBJECT (self), DBUS_PATH_TAG);
         if (dbus_path) {
